###############################################################################
## Flatfile/MySQL Database API 2.2.4
## KobazDatabaseAPI.tcl
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazDatabase.tcl
##           KobazConfig.tc
##           KobazSQL-Lib.tcl
##
## PURPOSE: Provide a method to access data files via kobazDB, mysql,
##          or dirDB - transparent to the script using this
##
## COMMANDS:
##
## DCC:
## .kbconfig KobazDatabaseAPI        - List possible global configuarion values
##
###############################################################################

###############################################################################
# All Configuration is done via the bot now: the .kbconfig KobazDatabaseAPI 
# command
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazDBAPI {
set a KobazDatabaseAPI

############################
# Settings - Do NOT Modify #
############################
proc add_configs {} {
variable a
add_config $a CREATE . . .
add_config $a "tog global" debug "0" {Enable debug? You probebly really don't need to turn this on}
add_config $a "0-2 global" dbtype "0" {Database type. 0=builtin 1=mysql 3=error loading mysql lib}
add_config $a "text global" dbdir "Kdb" {If dbtype is 0, the directory to store databases in}
add_config $a "text global" mysql_hostname "localhost" {If using mysql, host to connect to}
add_config $a "text global" mysql_username "" {If using mysql, username to connect with}
add_config $a "text global" mysql_password "" {If using mysql, password to connect with}
add_config $a "num_no0 global" mysql_delay "5" {If using mysql, delay in seconds to wait if a reconnect fails to allow another connect}
add_config $a "text global" mysql_database "KobazScripts" {If using mysql, database to use}
}
#################
# Settings -End #
#################

namespace export check_exists
namespace export dbClose
namespace export dbNumRecs
namespace export dbList
namespace export dbLoaded
namespace export dbCreate
namespace export dbRead
namespace export dbWrite
namespace export dbField
namespace export dbRecord
namespace export dbInsert
namespace export dbAppend
namespace export dbDelete
namespace export dbModify
namespace export dbSort
namespace export dbMatchingRecords
namespace export dbBinarySearch
namespace export dbFind

####
# Load required libraries

kbprovide $a
kbrequire Database
kbrequire Config
kbrequire SQL-Lib

# end
####

namespace import -force ::KobazSQL::*
namespace import -force ::KobazConfig::*

if {![info exists firstload]} {
    add_configs
    if {[catch {load_config $a}]} {
        setup_globals $a
    }
    set firstload  "1"
}

proc init {} {
    variable a

    if {[get_config KobazDatabaseAPI dbtype] == "3"} {
        set_config KobazDatabaseAPI dbtype 1
    }

    putlog "$a.tcl by Kobaz loaded."
}

proc dbtype {} {
    variable a
    set dbtype [get_config KobazDatabaseAPI dbtype]
    switch -- $dbtype {
        0 {set dbtype "builtin"}
        1 {
            if {(![sql_isconnected]) && ([timerexists KobazDBAPI::recon_delay] == "")} {
                timer [get_config KobazDatabaseAPI delay] KobazDBAPI::recon_delay
                if {![info exists ::KobazSQL::mytcl_have]} {return}
                if {$::KobazSQL::mytcl_have == "0"} {
                    putlog "You had an error loading mytcl.so, mysql will not be available"
                    set_config KobazDatabaseAPI dbtype 3
                    return
                }
                set host [get_config KobazDatabaseAPI mysql_hostname]
                set user [get_config KobazDatabaseAPI mysql_username]
                set pass [get_config KobazDatabaseAPI mysql_password]
                putlog "${a}.tcl: We appear to be disconnected from MySQL, reconnecting..."
                if {[catch {sql_connect $host $user $pass} error]} {
                    putlog "${a}.tcl: ERROR: $error"
                    return
                }
            }
            if {[sql_isconnected]} {set dbtype "mysql"}
        }
    }
    return $dbtype
}

proc recon_delay {} {return}

proc dbClose {alias} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbClose $alias]
        }
        mysql {
            return 0
        }
    }
    return
}

proc dbNumRecs {alias} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbNumRecs $alias]
        }
        mysql {            
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            return [sql_numrecs $db.$alias]
        }
    }
    return
}

proc check_exists {alias} {
    variable a

    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            set dir [get_config $a dbdir]
            if {[catch {check_dir_availability $dir} error]} {
                return -code error "Database dir not available: $error"
            }
            set filename [file join $dir $alias]
            return [file exists $filename]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            set tables [sql_query "SHOW TABLES FROM $db"]
            regsub -all {\.} $alias "_" alias
            if {[lsearch $tables $alias] != "-1"} {
                return 1
            } else {
                return 0
            }
        }
    }
    return 0
}


proc dbList {} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbList]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # just return a list of tables in current db
            return [sql_query "SHOW TABLES FROM $db"]
        }
    }
    return
}

proc dbLoaded {alias} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbLoaded $alias]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a database
            regsub -all {\.} $alias "_" alias

            set tables [sql_query "SHOW TABLES FROM $db"]
            if {[lsearch $tables $alias] != "-1"} {return 1} else {return 0}
        }
    }
    return
}

proc dbCreate {alias filename fieldlist {type ""}} {
    variable a

    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            set dir [get_config $a dbdir]
            if {[catch {check_dir_availability $dir} error]} {
                return -code error "Database dir not available: $error"
            }
            set filename [file join $dir $filename]
            return [KobazDB::dbCreate $alias $filename $fieldlist $type]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a database
            regsub -all {\.} $alias "_" alias
            catch {sql_drop_table $db.$alias}
            catch {sql_create_table $db.$alias}
            set createfields "recnum INT not NULL , primary key (recnum)"
            foreach field $fieldlist {
                set createfields "$createfields , $field TEXT"
            }
            return [sql_create_table $db.$alias "($createfields)"]
        }
    }
    return
}

proc dbRead {alias filename} {
    variable a

    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            set dir [get_config $a dbdir]
            set filename [file join $dir $filename]
            return [KobazDB::dbRead $alias $filename]
        }
        mysql {
            return 0
        }
    }
    return
}

proc dbWrite {alias} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbWrite $alias]
        }
        mysql {
            return 0
        }
    }
    return
}

proc dbField {alias field recnum {type ""}} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbField $alias $field $recnum]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias

            if {$type == "absolute"} {
                return [lindex [sql_getabsolutefield $db.$alias $field $recnum] 0]
            } else {
                set result  [sql_getfield $db.$alias $field $recnum]
                set result0 [lindex $result 0]
                if {[llength $result0] > 1} {
                    return $result0
                } else {
                    return $result
                }
            }
        }
    }
    return
}

proc dbFields {alias} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbFields $alias]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            set fields [sql_query "SHOW COLUMNS FROM $db.$alias"]

            set fieldlist ""
            foreach field $fields {
                set fieldlist "$fieldlist [lindex $field 0]"
            }

            return [lrange $fieldlist 1 end]
        }
    }
    return
}

proc dbFind {alias fieldname searchtext} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbFind $alias $fieldname $searchtext]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            return [KobazDBAPI::dbBinarySearch $alias $fieldname $searchtext]
        }
    }
    return
}

proc dbRecord {alias recnum} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbField $alias $field $recnum]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            return [lrange [lindex [sql_getrec $db.$alias $recnum] 0] 1 end]
        }
    }
    return
}

proc dbAppend {alias add} {
    set dbtype [dbtype]

    switch -- $dbtype {
        builtin {
            return [KobazDB::dbAppend $alias $add]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias

            set lastrec [sql_lastrec $db.$alias]

            if {$lastrec == ""} {
                set nextrec 0
            } else {
                set nextrec [expr $lastrec + 1]
            }

            set insertfields "$nextrec"
            foreach field $add {
                set field [sql_fixforadd $field]
                set insertfields "$insertfields , '$field'"
            }

           return [sql_exec "INSERT INTO $db.$alias VALUES ($insertfields)"]
        }
    }
    return
}

proc dbInsert {alias add} {
    set dbtype [dbtype]

    switch -- $dbtype {
        builtin {
            return [KobazDB::dbInsert $alias $add]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias

            set lastrec [sql_lastrec $db.$alias]

            if {$lastrec == ""} {
                set nextrec 0
            } else {
                set nextrec [expr $lastrec + 1]
            }

            set insertfields "$nextrec"
            foreach field $add {
                set field [sql_fixforadd $field]
                set insertfields "$insertfields , '$field'"
            }

           return [sql_exec "INSERT INTO $db.$alias VALUES ($insertfields)"]
        }
    }
    return
}

proc dbDelete {alias recnum} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbDelete $alias $recnum]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            return [sql_delete $db.$alias $recnum]
        }
    }
    return
}

proc dbModify {alias fieldname recnum data} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbModify $alias $fieldname $recnum $data]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            return [sql_modify $db.$alias $fieldname $recnum $data]
        }
    }
    return
}

proc dbSort {alias field {type ""}} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbSort $alias $field]
        }
    }
    return
}

proc dbMatchingRecords {alias field search} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbMatchingRecords $alias $field $search]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            return [sql_search $db.$alias $field $search]
        }
    }
    return
}

proc dbBinarySearch {alias field search} {
    set dbtype [dbtype]
    switch -- $dbtype {
        builtin {
            return [KobazDB::dbBinarySearch $alias $field $search]
        }
        mysql {
            set db [get_config KobazDatabaseAPI mysql_database]
            # alias is really a table
            regsub -all {\.} $alias "_" alias
            set search [sql_search $db.$alias $field $search]
            if {$search == ""} {set search "-1"}
            return $search
        }
    }
    return
}

init
}
