##############################################################################
## Kobaz Tools ver 2.2.4
## KobazTools.tcl
## 
## AUTHOR: Kobaz Mi'Daire <krobaz@yahoo.com>
## 
## REQUIRES: Eggdrop 1.3.x AND tcl 8.1
##           KobazConfig
##           KobazDatabaseAPI
##
## PURPOSE:  Provides various nifty tools used in most of my scripts
##
## FEATURES: Benchmark
##           Userlist flag checker
##           Find time since a [clock seconds]
##           Send messages over irc via different methods
##           Random number generator
##           Varous procs for doing boolean searches with KobazDatabase.tcl
## 
###############################################################################

############################################
############################################
## Modify contents below at your own risk ##
############################################
############################################

source scripts/kobazscripts/KobazDep.tcl
kbrequiretcl 8.1

namespace eval KobazTools {
set a KobazTools

namespace export start_bench
namespace export stop_bench
namespace export since
namespace export tellnick
namespace export parseterms
namespace export random
namespace export check_access
namespace export revorder
namespace export ridcolor
namespace export ldelete
namespace export botonchan

namespace export datafile_load
namespace export datafile_loadall
namespace export datafile_load_forchannel
namespace export datafile_save
namespace export datafile_expire
namespace export datafile_pub_join

####
# Load required libraries

kbprovide $a
kbrequire Config
kbrequire DatabaseAPI

# end
####

namespace import -force ::KobazDBAPI::*
namespace import -force ::KobazConfig::*

proc add_configs {} {
variable a
add_config $a CREATE . . .
add_config $a "tog global" debug "0" {Enable debug? You probebly really don't need to turn this on}
}

if {![info exists firstload]} {
    set lastvalue [expr [clock seconds] % 65536]

    add_configs
    if {[catch {load_config $a}]} {
        setup_globals $a
    }
    set firstload "1"
}

proc init {} {
    variable a

    bind dcc n kbflags KobazTools::dcc_kbflags
    bind dcc n kbdata  KobazTools::dcc_kbdata
    bind join - * KobazTools::datafile_pub_join

    set timername [timerexists KobazTools::datafile_expire]
    if {$timername == ""} {timer 10 KobazTools::datafile_expire}

    putlog "$a.tcl by Kobaz loaded."
}

proc debug {args} {
    variable a
    set debug [get_config $a debug]
    if {$debug == "1"} {
	putlog "TOOLS:DEBUG $args"
    }
}

proc dcc_kbflags {handle idx args} {
    putdcc $idx "The following flags are available:"
    putdcc $idx "-  -Everyone"
    putdcc $idx "*  -No One"
    putdcc $idx "c  -Channel status of nothing"
    putdcc $idx "-v -Channel status of +v (or lower)"
    putdcc $idx "v  -Channel status of +v (or higher)"
    putdcc $idx "o  -Channel status of +o"
    putdcc $idx "p  -Anyone that has partyline access"
    putdcc $idx "b  -Any user defined as a Bot"
    putdcc $idx "u  -**ANYONE** that is on the bot userlist"
    putdcc $idx "cm -Channel Master (or higher)"
    putdcc $idx "cn -Channel Owner  (or higher)"
    putdcc $idx "m  -Any global master  (or higher)"
    putdcc $idx "n  -Any global owner"
    putdcc $idx "Any user defined flag A-Z"
    putdcc $idx "Tips: You dont need to specify v and o for ops and voiced people to"
    putdcc $idx "have access, the v flag also includes the o flag, etc."
    putdcc $idx "Also, specifying * or - with other flags is pointless"
    return
}

proc error {error} {
    putlog "*** ERROR *** $error"
    return
}

proc ridcolor {text} {
    regsub -all {[0-9]?[0-9]?(,[0-9]?[0-9]?)?} $text "" text
    return $text
}

proc ldelete {list index} {return [lreplace $list $index $index]}

proc start_bench {} {
    global start_time
    set start_time [clock clicks]
}

proc stop_bench {} {
    global start_time

    set curr_time [clock clicks]
    set ms [expr $curr_time - $start_time]
    set ms [expr double($ms) / double(1000000)]
    set ms [format %.2f $ms]
    return "\[${ms}s\]"
}

proc revorder {list} {
    for {set x 0 ; set y [expr {[llength $list] -1}]} {$x < $y} \
	    {incr x ; incr y -1} {
	set t [lindex $list $x]
	set list [lreplace $list $x $x [lindex $list $y]]
	set list [lreplace $list $y $y $t]
    }
    return $list
}

proc check_access {nick channel handle flags {message ""}} {
    set x 0
    foreach flag $flags {
	set $x [single_check_access $nick $channel $handle $flag]
	set y [subst "$$x"]
	if {$y == "0"} {
	    if {$message == "*"} {
		return 0
	    }
	    tellnick notice $nick $message
	    return 0
	} else {
	    continue
	}
	incr x
    }
    
    return 1
}

proc single_check_access {nick channel handle flag} {
    if {![regexp {\-|\*|c|-v|v|o|p|b|cm|cn|m|n|[A-Z]} $flag]} {
	error "You have specified an illegal flag ($flag) for command access.  Not allowing access to command."
	return 0
    }

    set isvoice  [isvoice $nick $channel]
    set isop     [isop $nick $channel]
    set party    [matchattr $handle p]
    set onbot    [validuser $handle]
    set isbot    [matchattr $handle b]
    set chmaster [matchattr $handle |m $channel]
    set chowner  [matchattr $handle |n $channel]
    set master   [matchattr $handle m]
    set owner    [matchattr $handle n]
    
    #Return 1 if allowed,  Return 0 for deny

    if {[regexp {^[A-Z]$} $flag]} {return 1}

    switch -- $flag {
	* {return 0}
	- {return 1}
	c {if {!$isvoice && !$isop} {return 1}}
	v {if {$isop || $isvoice || $chmaster || $chowner || $master || $owner} {return 1}}
	-v {if {$isvoice && !$isop} {return 1}}
	o {if {$isop || $chmaster || $chowner || $master || $owner} {return 1}}
	p {if {$party} {return 1}}
	b {if {$isbot} {return 1}}
	u {if {$onbot} {return 1}}
	cm {if {$chmaster || $chowner || $master || $owner} {return 1}}
	cn {if {$chowner || $master || $owner} {return 1}}
	m {if {$master || $owner} {return 1}}
	n {if {$owner} {return 1}}
    }

    return 0
}

proc since {time} {
    set timesince [expr [clock seconds] - $time]
    set date ""

    if {[string index $timesince 0] == "-"} {
	# Time hasnt occured yet
	set timesince [string index $timesince 1 end]
    }


    # /60  =  minutes
    # /60  =  hours
    # /24  =  days
    #
    # /7   =  weeks
    # /365 =  years

    set timeleft $timesince

    set years [expr $timeleft / 60 / 60 / 24 / 365]
    if {$years != 0} {
	set date "${years}y"
	set timetaken [expr $years * 60 * 60 * 24 * 365]
	set timeleft  [expr $timeleft - $timetaken]
    }

    set weeks [expr $timeleft / 60 / 60 / 24 / 7]
    if {$weeks != 0} {
	set date "$date ${weeks}w"
	set timetaken [expr $weeks * 60 * 60 * 24 * 7]
	set timeleft  [expr $timeleft - $timetaken]
    }

    set days [expr $timeleft / 60 / 60 / 24]
    if {$days != 0} {
	set date "$date ${days}d"
	set timetaken [expr $days * 60 * 60 * 24]
	set timeleft  [expr $timeleft - $timetaken]
    }

    set hours [expr $timeleft / 60 / 60]
    if {$hours != 0} {
	set date "$date ${hours}h"
	set timetaken [expr $hours * 60 * 60]
	set timeleft  [expr $timeleft - $timetaken]
    }

    set min [expr $timeleft / 60]
    if {$min != 0} {
	set date "$date ${min}m"
	set timetaken [expr $min * 60]
	set timeleft  [expr $timeleft - $timetaken]
    }

    set sec $timeleft
    if {$sec != 0} {
	set date "$date ${sec}s"
    }

    return [string trim $date " "]

}

proc tellnick {type who {message ""}} {
    if {$message == ""} {return}
    if {$type == "notice"} {
	putserv "NOTICE $who :$message"
    } elseif {$type == "msg"} {
	putserv "PRIVMSG $who :$message"
    } elseif {$type == "dcc"} {
        putdcc $who $message
    } elseif {$type == "pub"} {
        puthelp "PRIVMSG $who :$message"
    } else {
        return
    }
    return
}

proc sort_slen {a b} {
    set la [string length $a]
    set lb [string length $b]
    if {$la > $lb} {return 1} elseif {$la < $lb} {return -1} {return 0}
}

proc parseterms {terms start max} {
    foreach i {must cant other} {
	array set $i {regexp {} {} {} before {} after {} by {} plain {}}
    }
    set ignored [list]
    while {[regexp -nocase \
	    {^(\+|-)?(([a-z]+):)?("([^"]+)"|([^ ]+))( +(.*)|$)} $terms \
	    full priority junk option word2 word junk junk rest]} {
        # "  -emacs color hilight fix
	set terms $rest
	if {[lsearch -exact [array names must] $option] == -1} {
	    lappend ignored [list $full "invalid option '$option'"]
	    continue
	}
	if {$word == ""} {set word $word2}
	switch -- $priority {+ {set var must} - {set var cant} {} {set var \
		other}}
	lappend ${var}($option) $word
	lappend search($var) [expr {$option == "" ? "" : "$option:"}][expr \
		{[string first { } $word] == -1 ? $word : "\"$word\""}]
    }
    set other() [lsort -command sort_slen $other()]
    set matches [list]
    set len [dbNumRecs quotes]
    for {set quoteid 0} {$quoteid < $len} {incr quoteid} {
	set break 0
	foreach i [array names must] {
	    foreach j $must($i) {
		if ![matchby $quoteid $i $j] {
		    set break 1;break
		}
	    }
	    if {$break} break
	}
	if {$break} continue
	foreach i [array names cant] {
	    foreach j $cant($i) {
		if [matchby $quoteid $i $j] {set break 1;break}
	    }
	    if {$break} break
	}
	if {$break} continue
	set ok 0
	set no 0
	foreach i [array names other] {
	    foreach j $other($i) {
		if [matchby $quoteid $i $j] {incr ok;set break 1;break} \
			{set no 1}
	    }
	    if {$break} break
	}
	if {!$ok && $no} continue
	lappend matches $quoteid
	if {[llength $matches] == $max+$start+1} break
    }
    set matches [lrange $matches $start end]
    foreach i $ignored {echo "ignored: [lindex $i 0] because: [lindex $i 1]"}
    return [list [array get search] $matches]
    concat [array get search] [list $matches]
}

proc matchby {quotenum type terms} {
    switch -- $type {
	regexp {
	    if [catch {regexp -nocase $terms [dbField quotes quote \
		    $quotenum]} bool] {return 0} {return $bool}
	}
	{} {
	    if [catch {string match -nocase *$terms* [dbField quotes quote \
		    $quotenum]} bool] {return 0} {return $bool}
	}
	plain {
	    expr {[string first [string tolower $terms] [string tolower \
		    [dbField quotes quote $quotenum]]] != -1}
	}
	before {
	    if [catch {clock scan $terms} time] {return 0} {expr {[dbField \
		    quotes date $quotenum] < $time}}
	}
	after {
	    if [catch {clock scan $terms} time] {return 0} {expr {[dbField \
		    quotes date $quotenum] > $time}}
	}
	by {
	    regexp -nocase {^([^!@]*)(!([^!@]*)(@(.*))?)?$} $terms junk nick \
		    junk user junk host
	    set nick [expr {$nick == "" ? "*" : $nick}]
	    set user [expr {$nick == "" ? "*" : $user}]
	    set host [expr {$nick == "" ? "*" : $host}]
	    string match -nocase $nick!$user@$host [dbField quotes author \
		    $quotenum]
	}
    }
}

proc rawrand {} {
    # per Knuth 3.6:
    # 65277 mod 8 = 5 (since 65536 is a power of 2)
    # c/m = .5-(1/6)\sqrt{3}
    # c = 0.21132*m = 13849, and should be odd.
    set KobazTools::lastvalue [expr (65277*$KobazTools::lastvalue+13849)%65536]
    set KobazTools::lastvalue [expr ($KobazTools::lastvalue+65536)%65536]
    return $KobazTools::lastvalue
}

proc random {base} {
    if {$base == ""} {return}
    set rr [KobazTools::rawrand]
    return [expr ($rr*$base)/65536]
}

proc botonchan {channel} {
    set channel [string tolower $channel]
    if {[lsearch -exact [string tolower [channels]] $channel] != "-1"} {
        return 1
    } else {
        return 0
    }
}

proc datafile_load {name channel {reload ""}} {
    if {[get_config KobazDatabaseAPI dbtype] == "3"} {
        return -code error "There was an error loading mytcl.so, $name commands not available"
    }

    set fields [get_config $name fields]

    variable chaninfo
    variable channels

    if {![KobazConfig::configname_channel_list $channel $name]} {
        if {[catch {load_config ${name}_$channel} error]} {
            putlog $error
        }
        setup_configs $name $channel
    }

    set file [get_config ${name}_$channel datafile]
    if {$file == ""} {
        if {[get_config ${name}_$channel enable] == "0"} {return}
        set_config ${name}_$channel enable 0
        putlog "${name}.tcl: No data file set for $channel. Disableing $name commands for this channel."
        return
    }

    set channel [string tolower $channel]

    if {[get_config ${name}_$channel enable] == 0} {return}
    if {[dbLoaded $file]} {set loaded 1} else {set loaded 0}
    set ms 0

    if {(!$loaded) || ($reload == "reload")} {
        if {![check_exists $file]} {
            putlog "${name}.tcl: File $file did not exist, creating."
            if {[catch {dbCreate $file $file $fields} error]} {
                return -code error "$error"
            }
            dbSort $file [lindex $fields 0]
            dbWrite $file
        } else {
            start_bench
            if {[catch {dbRead $file $file} error]} {
                stop_bench
                return -code error "$error"
            }
            dbSort $file [lindex $fields 0]
            set ms [stop_bench]
        }
        lappend chaninfo($file) $channel
    }

    catch {${name}::_datafile_load $file $channel} error
    debug "$name $channel: datafile_load callback: $error"

    return "$loaded $ms $file"
}

proc datafile_loadall {name {reload ""}} {
    foreach channel [channels] {
	set channel [string tolower $channel]

        set data ""
        catch {set data [datafile_load $name $channel $reload]} error
        if {$data == ""} {continue}

        set     file                 [lindex $data 2]
        set     alreadyloaded($file) [lindex $data 0]
        set     ms($file)            [lindex $data 1]
        lappend info($file)          $channel
        set     numrecs($file)       [dbNumRecs $file]
    }        

    foreach file [array names info] {
	if {[llength $info($file)] == "1"} {set s  ""} else {set s  "s"}
        if {$numrecs($file)        == "1"} {set ss ""} else {set ss "s"}

	if {$alreadyloaded($file)} {
	    putlog "${name}.tcl: $file for channel$s: $info($file) active.\
                    [dbNumRecs $file] Total record$ss."
	} else {
	    putlog "${name}.tcl: $file for channel$s: $info($file) loaded in $ms($file).\
                    [dbNumRecs $file] Total record$ss."
	}
    }

    return
}

proc datafile_load_forchannel {name channel} {
    if {[catch {set data [datafile_load $name $channel]} error]} {
        return -code error $error
    }

    if {$data == ""} {return}

    set file          [lindex $data 2]
    set alreadyloaded [lindex $data 0]
    set ms            [lindex $data 1]
    set numrecs       [dbNumRecs $file]

    if {($numrecs > "1") || ($numrecs == "0")} {set s "s"} else {set s ""}

    if {$alreadyloaded == 0} {
	putlog "${name}.tcl: $file for channel: $channel loaded in $ms. $numrecs Total record$s."
    }

    catch {${name}::_datafile_load_forchannel $channel} error
    debug "$name $channel: datafile_load_forchannel callback: $error"

    return
}

proc datafile_save {name file {sort ""} } {
    if {$sort == "sort"} { 
        set field1 [lindex [KobazConfig::get_config $name fields] 0]
        dbSort $file $field1
        debug "$name: Sorted database: $file"
    }

    if {[catch {dbWrite $file} error]} {
        error "$name: error saving file $file: $error"
        return
    }
    debug "$name: Wrote database: $file"

    catch {${name}::_datafile_save $file $sort} error
    debug "$name $file: datafile_save callback: $error"

    return
}

proc datafile_expire {} {
    variable chaninfo
    variable a
    
    foreach file [array names chaninfo] {
        foreach channel $chaninfo($file) {
            if {![botonchan $channel]} {
                set index [lsearch $chaninfo($file) $channel]
                set chaninfo($file) [ldelete $chaninfo($file) $index]
            }
        }
        if {[llength $chaninfo($file)] == "0"} {
            dbClose $file
            unset chaninfo($file)
            putlog "${a}.tcl: Closing unused data file: $file"
            catch {${name}::_datafile_expire $file} error
            debug "$name $file: datafile_expire callback: $error"
        }
    }

    set timername [timerexists KobazTools::datafile_expire]
    if {$timername == ""} {timer 10 KobazTools::datafile_expire}
}

proc datafile_pub_join {nick uhost handle channel} {
    global botnick
    if {$nick != $botnick} {return}

    set channel [string tolower $channel]
    foreach name [KobazConfig::configname_listscripts] {
        if {[get_config $name dojoin] != "1"} {continue}

        if {[catch {datafile_load_forchannel $name $channel} error]} {putlog $error}
        catch {${name}::_datafile_pub_join $channel} error
        debug "$name $channel: datafile_pub_join callback: $error"
    }

    return
}

proc dcc_kbdata {handle idx arg} {
    variable a

    if {[llength $arg] == "0"} {
        putdcc $idx "Options:"
        putdcc $idx "reload <name> #channel -If using builtin db, reload <name> database for #channel"
        putdcc $idx "reload <name> all      -If using builtin db, reload <name> databases for all channels "
        putdcc $idx "reconnect              -If using mysql force reconnect to database server"
        putdcc $idx "Example: .kbdata reload KobazQuote #lobby"
        return
    }
    
    set command [lindex $arg 0]

    switch -- $command {
        reload {
            set name [index $arg 1]
            set chanfl [lindex $arg 2]
            if {[get_config $name dojoin] == "0"} {
                # if dojoin is 0 then this script dosn't have a
                # datafile
                putdcc $idx "$name does not use datafiles"
                return
            }
            if {[get_config $name dojoin] == "0"} {
                
            }
            if {[string index $arg2 0] == "#"} {
                set name [lindex $arg 1]
                set channel $arg2
                set file [get_config ${a}_$channel datafile]
                putdcc $idx "Reloading datafile $file for $channel"
                datafile_load $a $channel reload
                return
            }
            if {$arg2 == "all"} {
                putlog "${a}.tcl: ReLoading Quotefile(s)..."
                putdcc $idx "Reloading all data files..."
                datafile_loadall reload
                return
            }
            putdcc $idx "Invalid command usage, type .KobazQuote for info"
            return
        }
        reconnect {
            putdcc $idx "Reconnecting not implemented"
        }
    }

    putdcc $idx "Invalid command usage, type .KobazQuote for info"

    return
}

init
}
